// Copyright (C)2004 Dimax ( http://www.xdimax.com )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#ifndef I2C_BRIDGE_H_10DF0DED_E85F_4f14_88D9_610BEA2211F7

#define I2C_BRIDGE_H_10DF0DED_E85F_4f14_88D9_610BEA2211F7

#ifdef __linux__
/*
* Compatibility stuff
*/
#include <sys/types.h>
typedef int         BOOL;        
typedef u_int8_t    BYTE;
typedef u_int8_t    UCHAR;
typedef u_int16_t   WORD; 
typedef u_int32_t   DWORD; 
typedef u_int32_t   ULONG; 
typedef void*       HANDLE;
#define _stdcall	/* nothing */
#define INVALID_HANDLE_VALUE ((HANDLE)(-1))
#endif

#pragma pack(push, 1)

typedef enum 
{
	U2C_SUCCESS, 
	U2C_BAD_PARAMETER,
	U2C_HARDWARE_NOT_FOUND,
	U2C_SLAVE_DEVICE_NOT_FOUND,
	U2C_TRANSACTION_FAILED,
	U2C_SLAVE_OPENNING_FOR_WRITE_FAILED, 
	U2C_SLAVE_OPENNING_FOR_READ_FAILED, 
	U2C_SENDING_MEMORY_ADDRESS_FAILED, 
	U2C_SENDING_DATA_FAILED,  
	U2C_NOT_IMPLEMENTED,  
	U2C_NO_ACK, 
    U2C_DEVICE_BUSY,
    U2C_MEMORY_ERROR,
	U2C_UNKNOWN_ERROR,
} U2C_RESULT;



typedef struct _U2C_TRANSACTION
{
	BYTE nSlaveDeviceAddress;
	BYTE nMemoryAddressLength; // can be from 0 up to 4 bytes
	DWORD nMemoryAddress;
	unsigned short nBufferLength; // can be from 1 up to 256
	BYTE Buffer[256];
} U2C_TRANSACTION, *PU2C_TRANSACTION;


// I2C bus frequency values:
const BYTE U2C_I2C_FREQ_FAST  = 0;
const BYTE U2C_I2C_FREQ_STD   = 1;
const BYTE U2C_I2C_FREQ_83KHZ = 2;
const BYTE U2C_I2C_FREQ_71KHZ = 3;
const BYTE U2C_I2C_FREQ_62KHZ = 4;
const BYTE U2C_I2C_FREQ_50KHZ = 6;
const BYTE U2C_I2C_FREQ_25KHZ = 16;
const BYTE U2C_I2C_FREQ_10KHZ = 46;
const BYTE U2C_I2C_FREQ_5KHZ  = 96;
const BYTE U2C_I2C_FREQ_2KHZ  = 242;


typedef struct _U2C_SLAVE_ADDR_LIST
{
	BYTE nDeviceNumber;
	BYTE List[256];
} U2C_SLAVE_ADDR_LIST, *PU2C_SLAVE_ADDR_LIST;

typedef struct _U2C_VERSION_INFO
{
	WORD MajorVersion;
	WORD MinorVersion;
} U2C_VERSION_INFO, *PU2C_VERSION_INFO;

typedef enum 
{
	LS_RELEASED,
	LS_DROPPED_BY_I2C_BRIDGE,
	LS_DROPPED_BY_SLAVE,
	LS_RAISED_BY_I2C_BRIDGE,
} U2C_LINE_STATE;


#pragma pack(pop)
// I2CBridge Board Initialization Routines
BYTE _stdcall U2C_GetDeviceCount();
U2C_RESULT _stdcall U2C_GetSerialNum(HANDLE hDevice, long* pSerialNum); 
U2C_RESULT _stdcall U2C_IsHandleValid(HANDLE hDevice); 
HANDLE _stdcall U2C_OpenDevice(BYTE nDevice);
HANDLE _stdcall U2C_OpenDeviceBySerialNum(long nSerialNum); 
U2C_RESULT _stdcall U2C_CloseDevice(HANDLE hDevice);
U2C_RESULT _stdcall U2C_GetFirmwareVersion(HANDLE hDevice, PU2C_VERSION_INFO pVersion);
U2C_RESULT _stdcall U2C_GetDriverVersion(HANDLE hDevice, PU2C_VERSION_INFO pVersion);
U2C_VERSION_INFO _stdcall U2C_GetDllVersion(); 

// I2C high level and configuration routines
U2C_RESULT _stdcall U2C_SetI2cFreq(HANDLE hDevice, BYTE Frequency);
U2C_RESULT _stdcall U2C_GetI2cFreq(HANDLE hDevice, BYTE *pFrequency);
U2C_RESULT _stdcall U2C_Read(HANDLE hDevice, PU2C_TRANSACTION pTransaction);
U2C_RESULT _stdcall U2C_Write(HANDLE hDevice, PU2C_TRANSACTION pTransaction);
U2C_RESULT _stdcall U2C_ScanDevices(HANDLE hDevice, PU2C_SLAVE_ADDR_LIST pList);

// I2C low level routines
U2C_RESULT _stdcall U2C_Start(HANDLE hDevice);
U2C_RESULT _stdcall U2C_RepeatedStart(HANDLE hDevice);
U2C_RESULT _stdcall U2C_Stop(HANDLE hDevice);
U2C_RESULT _stdcall U2C_PutByte(HANDLE hDevice, BYTE Data);
U2C_RESULT _stdcall U2C_GetByte(HANDLE hDevice, BYTE *pData);
U2C_RESULT _stdcall U2C_PutByteWithAck(HANDLE hDevice, BYTE Data);
U2C_RESULT _stdcall U2C_GetByteWithAck(HANDLE hDevice, BYTE *pData, BOOL bAck);
U2C_RESULT _stdcall U2C_PutAck(HANDLE hDevice, BOOL bAck);
U2C_RESULT _stdcall U2C_GetAck(HANDLE hDevice);

// I2c wire level routines
U2C_RESULT _stdcall U2C_ReadScl(HANDLE hDevice, U2C_LINE_STATE *pState);
U2C_RESULT _stdcall U2C_ReadSda(HANDLE hDevice, U2C_LINE_STATE *pState);
U2C_RESULT _stdcall U2C_ReleaseScl(HANDLE hDevice);
U2C_RESULT _stdcall U2C_ReleaseSda(HANDLE hDevice);
U2C_RESULT _stdcall U2C_DropScl(HANDLE hDevice);
U2C_RESULT _stdcall U2C_DropSda(HANDLE hDevice);

// GPIO routines
U2C_RESULT _stdcall U2C_SetIoDirection(HANDLE hDevice, ULONG Value, ULONG Mask);
U2C_RESULT _stdcall U2C_GetIoDirection(HANDLE hDevice, ULONG *pValue);
U2C_RESULT _stdcall U2C_IoWrite(HANDLE hDevice, ULONG Value, ULONG Mask);
U2C_RESULT _stdcall U2C_IoRead(HANDLE hDevice, ULONG *pValue);
U2C_RESULT _stdcall U2C_SetSingleIoDirection(HANDLE hDevice, ULONG IoNumber, BOOL bOutput);
U2C_RESULT _stdcall U2C_GetSingleIoDirection(HANDLE hDevice, ULONG IoNumber, BOOL *pbOutput);
U2C_RESULT _stdcall U2C_SingleIoWrite(HANDLE hDevice, ULONG IoNumber, BOOL Value);
U2C_RESULT _stdcall U2C_SingleIoRead(HANDLE hDevice, ULONG IoNumber, BOOL *pValue);


#endif //I2C_BRIDGE_H_10DF0DED_E85F_4f14_88D9_610BEA2211F7
